use actix_web::{web, HttpResponse, Responder};
use askama_actix::TemplateToResponse;
use serde::Deserialize;
use sqlx::PgPool;

use crate::{
    endpoints::{availability::NewOrEditAvailabilityTemplate, IdPath},
    models::{Availabillity, User},
};

#[derive(Deserialize)]
struct EditAvailabilityQuery {
    #[serde(rename(deserialize = "wholeday"))]
    whole_day: Option<bool>,
}

#[actix_web::get("/availabillity/edit/{id}")]
pub async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
    query: web::Query<EditAvailabilityQuery>,
) -> impl Responder {
    if let Ok(availabillity) = Availabillity::read_by_id(pool.get_ref(), path.id).await {
        if availabillity.user_id == user.id {
            let start_time = availabillity
                .start_time
                .and_then(|d| Some(d.format("%R").to_string()));

            let end_time = availabillity
                .end_time
                .and_then(|d| Some(d.format("%R").to_string()));

            let has_time = availabillity.start_time.is_some() && availabillity.end_time.is_some();

            let template = NewOrEditAvailabilityTemplate {
                user: user.into_inner(),
                date: availabillity.date,
                whole_day: query.whole_day.unwrap_or(!has_time),
                id: Some(path.id),
                start_time: start_time.as_deref(),
                end_time: end_time.as_deref(),
                comment: availabillity.comment.as_deref(),
            };

            return template.to_response();
        }
    }

    HttpResponse::BadRequest().body("Availabillity with this id doesn't exist.")
}
