use actix_identity::Identity;
use actix_web::{web, HttpResponse, Responder};
use askama::Template;
use askama_actix::TemplateToResponse;
use chrono::NaiveDate;
use sqlx::PgPool;

use crate::{endpoints::NaiveDateQuery, models::{Role, User, Location}};

#[derive(Template)]
#[template(path = "events/new.html")]
pub struct NewEventTemplate {
    user: User,
    date: NaiveDate,
    locations: Vec<Location>
}

#[actix_web::get("/events/new")]
pub async fn get(user: Identity, pool: web::Data<PgPool>, query: web::Query<NaiveDateQuery>) -> impl Responder {
    let current_user = User::read_by_id(pool.get_ref(), user.id().unwrap().parse().unwrap())
        .await
        .unwrap();

    if current_user.role != Role::Admin && current_user.role != Role::AreaManager {
        return HttpResponse::Unauthorized().finish();
    }

    let locations;
    if current_user.role == Role::Admin {
        locations = Location::read_all_including_area(pool.get_ref()).await.unwrap();
    } else {
        locations = Location::read_by_area(pool.get_ref(), current_user.area_id).await.unwrap();
    }

    let template = NewEventTemplate { user: current_user, date: query.date, locations };

    return template.to_response();
}
